@@ -9,12 +9,13 @@ from py_expression_eval import Parser
 
 from massql import msql_parser
 from massql import msql_fileloading
+from massql import msql_engine_filters
+from massql.msql_engine_filters import _get_mz_tolerance, _get_minintensity
 
 math_parser = Parser()
 console = logging.StreamHandler()
 console.setLevel(logging.INFO)
 
-
 def DEBUG_MSG(msg):
     import sys
 
@@ -45,134 +46,27 @@ def _get_da_tolerance(qualifiers):
 
     return None
 
-def _get_mz_tolerance(qualifiers, mz):
-    if qualifiers is None:
-        return 0.1
 
-    if "qualifierppmtolerance" in qualifiers:
-        ppm = qualifiers["qualifierppmtolerance"]["value"]
-        mz_tol = abs(ppm * mz / 1000000)
-        return mz_tol
-
-    if "qualifiermztolerance" in qualifiers:
-        return qualifiers["qualifiermztolerance"]["value"]
-
-    return 0.1
-
-def _get_minintensity(qualifier):
+def process_query(input_query, input_filename, path_to_grammar=None, cache=True, parallel=False, ms1_df=None, ms2_df=None):
     """
-    Returns absolute min and relative min
+    Process an actual query
 
     Args:
-        qualifier ([type]): [description]
+        input_query ([type]): [description]
+        input_filename ([type]): [description]
+        path_to_grammar ([type], optional): [description]. Defaults to None.
+        cache (bool, optional): [description]. Defaults to True.
+        parallel (bool, optional): [description]. Defaults to False.
+        ms1_df ([type], optional): [description]. Defaults to None. Pass in if you have these data in memory
+        ms2_df ([type], optional): [description]. Defaults to None. Pass in if you have these data in memory
 
     Returns:
-        [type]: [description]
+        query results data frame: [description]
     """
 
-    min_intensity = 0
-    min_percent_intensity = 0
-    min_tic_percent_intensity = 0
-    
-
-    if qualifier is None:
-        min_intensity = 0
-        min_percent_intensity = 0
-
-        return min_intensity, min_percent_intensity, min_tic_percent_intensity
-    
-    if "qualifierintensityvalue" in qualifier:
-        min_intensity = float(qualifier["qualifierintensityvalue"]["value"])
-
-    if "qualifierintensitypercent" in qualifier:
-        min_percent_intensity = float(qualifier["qualifierintensitypercent"]["value"]) / 100
-
-    if "qualifierintensityticpercent" in qualifier:
-        min_tic_percent_intensity = float(qualifier["qualifierintensityticpercent"]["value"]) / 100
-
-    # since the subsequent comparison is a strict greater than, if people set it to 100, then they won't get anything. 
-    min_percent_intensity = min(min_percent_intensity, 0.99)
-
-    return min_intensity, min_percent_intensity, min_tic_percent_intensity
-
-def _get_intensitymatch_range(qualifiers, match_intensity):
-    """
-    Matching the intensity range
-
-    Args:
-        qualifiers ([type]): [description]
-        match_intensity ([type]): [description]
-
-    Returns:
-        [type]: [description]
-    """
-
-    min_intensity = 0
-    max_intensity = 0
-
-    if "qualifierintensitytolpercent" in qualifiers:
-        tolerance_percent = qualifiers["qualifierintensitytolpercent"]["value"]
-        tolerance_value = float(tolerance_percent) / 100 * match_intensity
-        
-        min_intensity = match_intensity - tolerance_value
-        max_intensity = match_intensity + tolerance_value
-
-    return min_intensity, max_intensity
-
-
-
-def _filter_intensitymatch(ms_filtered_df, register_dict, condition):
-    if "qualifiers" in condition:
-        if "qualifierintensitymatch" in condition["qualifiers"] and \
-            "qualifierintensitytolpercent" in condition["qualifiers"]:
-            qualifier_expression = condition["qualifiers"]["qualifierintensitymatch"]["value"]
-            qualifier_variable = qualifier_expression[0] #TODO: This assumes the variable is the first character in the expression, likely a bad assumption
-
-            grouped_df = ms_filtered_df.groupby("scan").sum().reset_index()
-
-            filtered_grouped_scans = []
-            for grouped_scan in grouped_df.to_dict(orient="records"):
-                # Reading from the register
-                key = "scan:{}:variable:{}".format(grouped_scan["scan"], qualifier_variable)
-
-                if key in register_dict:
-                    register_value = register_dict[key]                    
-                    evaluated_new_expression = math_parser.parse(qualifier_expression).evaluate({
-                        qualifier_variable : register_value
-                    })
-
-                    min_match_intensity, max_match_intensity = _get_intensitymatch_range(condition["qualifiers"], evaluated_new_expression)
-
-                    scan_intensity = grouped_scan["i"]
-
-                    #print(key, scan_intensity, qualifier_expression, min_match_intensity, max_match_intensity, grouped_scan)
-
-                    if scan_intensity > min_match_intensity and \
-                        scan_intensity < max_match_intensity:
-                        filtered_grouped_scans.append(grouped_scan)
-                else:
-                    # Its not in the register, which means we don't find it
-                    continue
-            return pd.DataFrame(filtered_grouped_scans)
-
-    return ms_filtered_df
-
-def _set_intensity_register(ms_filtered_df, register_dict, condition):
-    if "qualifiers" in condition:
-        if "qualifierintensityreference" in condition["qualifiers"]:
-            qualifier_variable = condition["qualifiers"]["qualifierintensitymatch"]["value"]
-
-            grouped_df = ms_filtered_df.groupby("scan").sum().reset_index()
-            for grouped_scan in grouped_df.to_dict(orient="records"):
-                # Saving into the register
-                key = "scan:{}:variable:{}".format(grouped_scan["scan"], qualifier_variable)
-                register_dict[key] = grouped_scan["i"]
-    return
-
-def process_query(input_query, input_filename, path_to_grammar=None, cache=True, parallel=False):
     parsed_dict = msql_parser.parse_msql(input_query, path_to_grammar=path_to_grammar)
 
-    return _evalute_variable_query(parsed_dict, input_filename, cache=cache, parallel=parallel)
+    return _evalute_variable_query(parsed_dict, input_filename, cache=cache, parallel=parallel, ms1_df=ms1_df, ms2_df=ms2_df)
 
 def _determine_mz_max(mz, ppm_tol, da_tol):
     da_tol = da_tol if da_tol < 10000 else 0
@@ -184,7 +78,7 @@ def _determine_mz_max(mz, ppm_tol, da_tol):
 
     return mz + half_delta
 
-def _evalute_variable_query(parsed_dict, input_filename, cache=True, parallel=False):
+def _evalute_variable_query(parsed_dict, input_filename, cache=True, parallel=False, ms1_df=None, ms2_df=None):
     # Lets check if there is a variable in here, the only one allowed is X
     for condition in parsed_dict["conditions"]:
         try:
@@ -228,9 +122,14 @@ def _evalute_variable_query(parsed_dict, input_filename, cache=True, parallel=Fa
 
     # Here we will check if there is a variable in the expression
     for condition in parsed_dict["conditions"]:
+        if not "value" in condition:
+            # If there is no value in the condition, e.g. retention time and mobility
+            continue
+        
         for value in condition["value"]:
             try:
                 # Checking if X is in any string
+                # TODO: Check multiple conditions, to see if any of them have an X, e.g. (101 OR X-10)
                 if "X" in value[0]:
                     if value == "X":
                         # This is the main varaible, not expression containing it
@@ -258,7 +157,9 @@ def _evalute_variable_query(parsed_dict, input_filename, cache=True, parallel=Fa
                 # This is when the target is actually a float
                 pass
 
-    ms1_df, ms2_df = msql_fileloading.load_data(input_filename, cache=cache)
+    # Loading data if not passed in 
+    if ms1_df is None:
+        ms1_df, ms2_df = msql_fileloading.load_data(input_filename, cache=cache)
 
     # Here we are going to translate the variable query into a concrete query based upon the data
     all_concrete_queries = []
@@ -277,6 +178,7 @@ def _evalute_variable_query(parsed_dict, input_filename, cache=True, parallel=Fa
                     pass
                 non_variable_conditions.append(condition)
         presearch_parse["conditions"] = non_variable_conditions
+
         ms1_df, ms2_df = _executeconditions_query(presearch_parse, input_filename, cache=cache)
         variable_x_ms1_df = ms1_df
 
@@ -307,6 +209,7 @@ def _evalute_variable_query(parsed_dict, input_filename, cache=True, parallel=Fa
 
         masses_considered_df = pd.DataFrame()
         masses_considered_df["mz"] = pd.concat(masses_considered_df_list)
+        # NOTE: This might cause bugs, we might consider every mass within every single scan, or at least we could make the tolernace as even smaller than half the max
         masses_considered_df["mz_max"] = masses_considered_df["mz"].apply(lambda x: _determine_mz_max(x, variable_properties["ppm_tolerance"], variable_properties["da_tolerance"]))
         
         masses_considered_df = masses_considered_df.sort_values("mz")
@@ -495,137 +398,40 @@ def _executeconditions_query(parsed_dict, input_filename, ms1_input_df=None, ms2
             ms1_scans = set(ms2_df["ms1scan"])
             ms1_df = ms1_df[ms1_df["scan"].isin(ms1_scans)]
 
-    # These are for the WHERE clause
+        # Mobility Filters
+        if condition["type"] == "mobilitycondition":
+            min_mobility = condition["min"]
+            max_mobility = condition["max"]
+
+            if "mobility" in ms2_df:
+                ms2_df = ms2_df[(ms2_df["mobility"] >= min_mobility) & (ms2_df["mobility"] <= max_mobility)]
+
+            if "mobility" in ms1_df.columns:
+                ms1_df = ms1_df[(ms1_df["mobility"] >= min_mobility) & (ms1_df["mobility"] <= max_mobility)]
+
+    # These are for the WHERE clause for peaks
     for condition in all_conditions:
         if not condition["conditiontype"] == "where":
             continue
 
-        #logging.error("WHERE CONDITION", condition)
-
         # Filtering MS2 Product Ions
         if condition["type"] == "ms2productcondition":
-            mz = condition["value"][0]
-            mz_tol = _get_mz_tolerance(condition.get("qualifiers", None), mz)
-            mz_min = mz - mz_tol
-            mz_max = mz + mz_tol
-
-            min_int, min_intpercent, min_tic_percent_intensity = _get_minintensity(condition.get("qualifiers", None))
-
-            ms2_filtered_df = ms2_df[(ms2_df["mz"] > mz_min) & 
-                                    (ms2_df["mz"] < mz_max) & 
-                                    (ms2_df["i"] > min_int) & 
-                                    (ms2_df["i_norm"] > min_intpercent) & 
-                                    (ms2_df["i_tic_norm"] > min_tic_percent_intensity)]
-
-            # Setting the intensity match register
-            _set_intensity_register(ms2_filtered_df, reference_conditions_register, condition)
-
-            # Applying the intensity match
-            ms2_filtered_df = _filter_intensitymatch(ms2_filtered_df, reference_conditions_register, condition)
-
-            if len(ms2_filtered_df) == 0:
-                # This means we've filtered everything out
-                ms2_df = pd.DataFrame()
-                ms1_df = pd.DataFrame()
-
-                return ms1_df, ms2_df
-            
-            # Filtering the actual data structures
-            filtered_scans = set(ms2_filtered_df["scan"])
-            ms2_df = ms2_df[ms2_df["scan"].isin(filtered_scans)]
-
-            # Filtering the MS1 data now
-            ms1_scans = set(ms2_df["ms1scan"])
-            ms1_df = ms1_df[ms1_df["scan"].isin(ms1_scans)]
-
+            ms1_df, ms2_df = msql_engine_filters.ms2prod_condition(condition, ms1_df, ms2_df, reference_conditions_register)
             continue
 
         # Filtering MS2 Precursor m/z
         if condition["type"] == "ms2precursorcondition":
-            mz = condition["value"][0]
-            mz_tol = _get_mz_tolerance(condition.get("qualifiers", None), mz)
-            mz_min = mz - mz_tol
-            mz_max = mz + mz_tol
-
-            ms2_df = ms2_df[(
-                ms2_df["precmz"] > mz_min) & 
-                (ms2_df["precmz"] < mz_max)
-            ]
-
-            # Filtering the MS1 data now
-            ms1_scans = set(ms2_df["ms1scan"])
-            ms1_df = ms1_df[ms1_df["scan"].isin(ms1_scans)]
-
+            ms1_df, ms2_df = msql_engine_filters.ms2prec_condition(condition, ms1_df, ms2_df, reference_conditions_register)
             continue
 
         # Filtering MS2 Neutral Loss
         if condition["type"] == "ms2neutrallosscondition":
-            mz = condition["value"][0]
-            mz_tol = _get_mz_tolerance(condition.get("qualifiers", None), mz) #TODO: This is incorrect logic if it comes to PPM accuracy
-            nl_min = mz - mz_tol
-            nl_max = mz + mz_tol
-
-            min_int, min_intpercent, min_tic_percent_intensity = _get_minintensity(condition.get("qualifiers", None))
-
-            ms2_filtered_df = ms2_df[
-                ((ms2_df["precmz"] - ms2_df["mz"]) > nl_min) & 
-                ((ms2_df["precmz"] - ms2_df["mz"]) < nl_max) &
-                (ms2_df["i"] > min_int) & 
-                (ms2_df["i_norm"] > min_intpercent) & 
-                (ms2_df["i_tic_norm"] > min_tic_percent_intensity)
-            ]
-
-            # Setting the intensity match register
-            _set_intensity_register(ms2_filtered_df, reference_conditions_register, condition)
-
-            # Applying the intensity match
-            ms2_filtered_df = _filter_intensitymatch(ms2_filtered_df, reference_conditions_register, condition)
-
-            # Filtering the actual data structures
-            filtered_scans = set(ms2_filtered_df["scan"])
-            ms2_df = ms2_df[ms2_df["scan"].isin(filtered_scans)]
-
-            # Filtering the MS1 data now
-            ms1_scans = set(ms2_df["ms1scan"])
-            ms1_df = ms1_df[ms1_df["scan"].isin(ms1_scans)]
-
+            ms1_df, ms2_df = msql_engine_filters.ms2nl_condition(condition, ms1_df, ms2_df, reference_conditions_register)
             continue
 
         # finding MS1 peaks
         if condition["type"] == "ms1mzcondition":
-            mz = condition["value"][0]
-            mz_tol = _get_mz_tolerance(condition.get("qualifiers", None), mz)
-            mz_min = mz - mz_tol
-            mz_max = mz + mz_tol
-
-            min_int, min_intpercent, min_tic_percent_intensity = _get_minintensity(condition.get("qualifiers", None))
-            ms1_filtered_df = ms1_df[
-                (ms1_df["mz"] > mz_min) & 
-                (ms1_df["mz"] < mz_max) & 
-                (ms1_df["i"] > min_int) & 
-                (ms1_df["i_norm"] > min_intpercent) & 
-                (ms1_df["i_tic_norm"] > min_tic_percent_intensity)]
-            
-            #print("YYY", mz_min, mz_max, min_int, min_intpercent, len(ms1_filtered_df))
-
-            # Setting the intensity match register
-            _set_intensity_register(ms1_filtered_df, reference_conditions_register, condition)
-
-            # Applying the intensity match
-            ms1_filtered_df = _filter_intensitymatch(ms1_filtered_df, reference_conditions_register, condition)
-
-            #print(ms1_filtered_df)
-
-            if len(ms1_filtered_df) == 0:
-                return pd.DataFrame(), pd.DataFrame()
-
-            # Filtering the actual data structures
-            filtered_scans = set(ms1_filtered_df["scan"])
-            ms1_df = ms1_df[ms1_df["scan"].isin(filtered_scans)]
-
-            if "ms1scan" in ms2_df:
-                ms2_df = ms2_df[ms2_df["ms1scan"].isin(filtered_scans)]
-
+            ms1_df, ms2_df = msql_engine_filters.ms1_condition(condition, ms1_df, ms2_df, reference_conditions_register)
             continue
 
         skip_conditions = [ "rtmincondition", 
@@ -633,7 +439,8 @@ def _executeconditions_query(parsed_dict, input_filename, ms1_input_df=None, ms2
                             "polaritycondition", 
                             "scanmincondition", 
                             "scanmaxcondition",
-                            "chargecondition"]
+                            "chargecondition",
+                            "mobilitycondition"]
         
         if condition["type"] in skip_conditions:
             continue
@@ -645,21 +452,23 @@ def _executeconditions_query(parsed_dict, input_filename, ms1_input_df=None, ms2
         if not condition["conditiontype"] == "filter":
             continue
 
-        logging.error("FILTER CONDITION", condition)
-
         # filtering MS1 peaks
         if condition["type"] == "ms1mzcondition":
+            if len(ms1_df) == 0:
+                continue
+
             mz = condition["value"][0]
             mz_tol = 0.1
             mz_min = mz - mz_tol
             mz_max = mz + mz_tol
             ms1_df = ms1_df[(ms1_df["mz"] > mz_min) & (ms1_df["mz"] < mz_max)]
 
-            print("FILTER", mz_min, mz_max, len(ms1_df))
-
             continue
         
         if condition["type"] == "ms2productcondition":
+            if len(ms2_df) == 0:
+                continue
+
             mz = condition["value"][0]
             mz_tol = _get_mz_tolerance(condition.get("qualifiers", None), mz)
             mz_min = mz - mz_tol
@@ -741,6 +550,9 @@ def _executecollate_query(parsed_dict, ms1_df, ms2_df):
             result_df = pd.DataFrame()
 
             if parsed_dict["querytype"]["datatype"] == "datams1data":
+                if len(ms1_df) == 0:
+                    return pd.DataFrame()
+
                 groupby_columns = ["scan"]
                 kept_columns = ["scan", "rt"]
 
@@ -748,6 +560,9 @@ def _executecollate_query(parsed_dict, ms1_df, ms2_df):
                     groupby_columns.append("comment")
                     kept_columns.append("comment")
 
+                if "mobility" in ms1_df:
+                    kept_columns.append("mobility")
+
                 result_df = ms1_df.groupby(groupby_columns).first().reset_index()
                 result_df = result_df[kept_columns]
                 result_df["mslevel"] = 1
@@ -757,6 +572,9 @@ def _executecollate_query(parsed_dict, ms1_df, ms2_df):
                 result_df["i"] = ms1sum_df["i"]
                 result_df["i_norm"] = ms1norm_df["i_norm"]
             if parsed_dict["querytype"]["datatype"] == "datams2data":
+                if len(ms2_df) == 0:
+                    return pd.DataFrame()
+
                 kept_columns = ["scan", "precmz", "ms1scan", "rt", "charge"]
                 groupby_columns = ["scan"]
 
@@ -764,6 +582,9 @@ def _executecollate_query(parsed_dict, ms1_df, ms2_df):
                     groupby_columns.append("comment")
                     kept_columns.append("comment")
 
+                if "mobility" in ms2_df:
+                    kept_columns.append("mobility")
+
                 result_df = ms2_df.groupby(groupby_columns).first().reset_index()
                 result_df = result_df[kept_columns]
 
