@@ -5,7 +5,11 @@ import pandas as pd
 import numpy as np
 from tqdm import tqdm
 from matchms.importing import load_from_mgf
-from pyteomics import mzxml
+from pyteomics import mzxml, mzml
+
+import logging
+logger = logging.getLogger('msql_fileloading')
+
 
 def load_data(input_filename, cache=False):
     """
@@ -37,7 +41,8 @@ def load_data(input_filename, cache=False):
     # Actually loading
     if input_filename[-5:].lower() == ".mzml":
         #ms1_df, ms2_df = _load_data_mzML(input_filename)
-        ms1_df, ms2_df = _load_data_mzML2(input_filename)
+        #ms1_df, ms2_df = _load_data_mzML2(input_filename) # Faster version using pymzML
+        ms1_df, ms2_df = _load_data_mzML_pyteomics(input_filename) # Faster version using pymzML
 
     elif input_filename[-6:].lower() == ".mzxml":
         ms1_df, ms2_df = _load_data_mzXML(input_filename)
@@ -47,6 +52,9 @@ def load_data(input_filename, cache=False):
     
     elif input_filename[-4:].lower() == ".mgf":
         ms1_df, ms2_df = _load_data_mgf(input_filename)
+
+    elif input_filename[-4:].lower() == ".txt":
+        ms1_df, ms2_df = _load_data_txt(input_filename)
     
     else:
         print("Cannot Load File Extension")
@@ -233,7 +241,7 @@ def _load_data_mzXML(input_filename):
 
 def _determine_scan_polarity_mzML(spec):
     """
-    Gets an enum for positive and negative polarity
+    Gets an enum for positive and negative polarity, for pymzml
 
     Args:
         spec ([type]): [description]
@@ -251,6 +259,25 @@ def _determine_scan_polarity_mzML(spec):
 
     return polarity
 
+def _determine_scan_polarity_pyteomics_mzML(spec):
+    """
+    Gets an enum for positive and negative polarity, for pyteomics
+
+    Args:
+        spec ([type]): [description]
+
+    Returns:
+        [type]: [description]
+    """
+    polarity = 0
+
+    if "negative scan" in spec:
+        polarity = 2
+    if "positive scan" in spec:
+        polarity = 1
+
+    return polarity
+
 def _determine_scan_polarity_mzXML(spec):
     polarity = 0
     if spec["polarity"] == "+":
@@ -259,6 +286,166 @@ def _determine_scan_polarity_mzXML(spec):
         polarity = 2
     return polarity
 
+def _load_data_mzML_pyteomics(input_filename):
+    """
+    This is a loading operation using pyteomics to help with loading mzML files with ion mobility
+
+    Args:
+        input_filename ([type]): [description]
+    """
+
+    previous_ms1_scan = 0
+
+    # MS1
+    all_mz = []
+    all_rt = []
+    all_polarity = []
+    all_i = []
+    all_i_norm = []
+    all_i_tic_norm = []
+    all_scan = []
+
+    # MS2
+    all_msn_mz = []
+    all_msn_rt = []
+    all_msn_polarity = []
+    all_msn_i = []
+    all_msn_i_norm = []
+    all_msn_i_tic_norm = []
+    all_msn_scan = []
+    all_msn_precmz = []
+    all_msn_ms1scan = []
+    all_msn_charge = []
+    all_msn_mobility = []
+
+    with mzml.read(input_filename) as reader:
+        for spectrum in tqdm(reader):
+            if len(spectrum["intensity array"]) == 0:
+                continue
+
+            # BUG: This retention time is ambiguous in terms of retention time units
+            # Filed bug with pyteomics: https://github.com/levitsky/pyteomics/issues/56
+            # Work around will to check if there is a retention greater than a certain value
+            # And then if so, we can assume seconds and then divide by 60 to get minutes
+            try:
+                rt = spectrum["scanList"]["scan"][0]["scan start time"]
+            except:
+                rt = 0
+
+            scan = int(spectrum["id"].split("scan=")[-1])
+                
+            mz = spectrum["m/z array"]
+            intensity = spectrum["intensity array"]
+            i_max = max(intensity)
+            i_sum = sum(intensity)
+
+            mslevel = spectrum["ms level"]
+            if mslevel == 1:
+                all_mz += list(mz)
+                all_i += list(intensity)
+                all_i_norm += list(intensity / i_max)
+                all_i_tic_norm += list(intensity / i_sum)
+                all_rt += len(mz) * [rt]
+                all_scan += len(mz) * [scan]
+                all_polarity += len(mz) * [_determine_scan_polarity_pyteomics_mzML(spectrum)]
+
+                previous_ms1_scan = scan
+
+            if mslevel == 2:
+                msn_mz = spectrum["precursorList"]["precursor"][0]["selectedIonList"]["selectedIon"][0]["selected ion m/z"]
+                msn_charge = 0
+
+                if "charge state" in spectrum["precursorList"]["precursor"][0]["selectedIonList"]["selectedIon"][0]:
+                    msn_charge = int(spectrum["precursorList"]["precursor"][0]["selectedIonList"]["selectedIon"][0]["charge state"])
+
+                all_msn_mz += list(mz)
+                all_msn_i += list(intensity)
+                all_msn_i_norm += list(intensity / i_max)
+                all_msn_i_tic_norm += list(intensity / i_sum)
+                all_msn_rt += len(mz) * [rt]
+                all_msn_scan += len(mz) * [scan]
+                all_msn_polarity += len(mz) * [_determine_scan_polarity_pyteomics_mzML(spectrum)]
+                all_msn_precmz += len(mz) * [msn_mz]
+                all_msn_ms1scan += len(mz) * [previous_ms1_scan] 
+                all_msn_charge += len(mz) * [msn_charge]
+
+                if "product ion mobility" in spectrum["precursorList"]["precursor"][0]["selectedIonList"]["selectedIon"][0]:
+                    mobility = spectrum["precursorList"]["precursor"][0]["selectedIonList"]["selectedIon"][0]["product ion mobility"]
+                    all_msn_mobility += len(mz) * [mobility]
+
+    ms1_df = pd.DataFrame()
+    if len(all_mz) > 0:
+        ms1_df['i'] = all_i
+        ms1_df['i_norm'] = all_i_norm
+        ms1_df['i_tic_norm'] = all_i_tic_norm
+        ms1_df['mz'] = all_mz
+        ms1_df['scan'] = all_scan
+        ms1_df['rt'] = all_rt
+        ms1_df['polarity'] = all_polarity
+
+    ms2_df = pd.DataFrame()
+    if len(all_msn_mz) > 0:
+        ms2_df['i'] = all_msn_i
+        ms2_df['i_norm'] = all_msn_i_norm
+        ms2_df['i_tic_norm'] = all_msn_i_tic_norm
+        ms2_df['mz'] = all_msn_mz
+        ms2_df['scan'] = all_msn_scan
+        ms2_df['rt'] = all_msn_rt
+        ms2_df["polarity"] = all_msn_polarity
+        ms2_df["precmz"] = all_msn_precmz
+        ms2_df["ms1scan"] = all_msn_ms1scan
+        ms2_df["charge"] = all_msn_charge
+
+        if len(all_msn_mobility) == len(all_msn_i):
+            ms2_df["mobility"] = all_msn_mobility
+
+    # HACK: This is a hack to get around the fact that pyteomics does not return the units for retention for scans
+    try:
+        MS_precisions = {
+            1: 5e-6,
+            2: 20e-6,
+            3: 20e-6,
+            4: 20e-6,
+            5: 20e-6,
+            6: 20e-6,
+            7: 20e-6,
+        }
+        run = pymzml.run.Reader(input_filename, MS_precisions=MS_precisions)
+
+        correct_to_min = False
+        for spec in run:
+            rt = spec.scan_time_in_minutes()
+            spectrum_scan = int(spec.ID)
+            
+            ms1_filtered_df = ms1_df[ms1_df["scan"] == spectrum_scan]
+            ms2_filtered_df = ms2_df[ms2_df["scan"] == spectrum_scan]
+
+            if len(ms1_filtered_df) > 0:
+                # compare the RT
+                rt_delta = abs(rt - ms1_filtered_df["rt"].iloc[0])
+                if rt_delta > 0.1:
+                    # This we assume means that its in seconds instead of minutes
+                    correct_to_min = True
+                break
+            elif len(ms2_filtered_df) > 0:
+                rt_delta = abs(rt - ms2_filtered_df["rt"].iloc[0])
+                if rt_delta > 0.1:
+                    # This we assume means that its in seconds instead of minutes
+                    correct_to_min = True
+                break
+            else:
+                continue
+            
+        # Here lets correct it
+        if correct_to_min:
+            logger.info("Correcting retention time seconds to minutes")
+            ms1_df["rt"] = ms1_df["rt"] / 60
+            ms2_df["rt"] = ms2_df["rt"] / 60
+    except:
+        pass
+    
+    return ms1_df, ms2_df
+
 def _load_data_mzML2(input_filename):
     """This is a faster loading version, but a bit more memory intensive
 
@@ -475,3 +662,28 @@ def _load_data_mzML(input_filename):
 
     return ms1_df, ms2_df
 
+def _load_data_txt(input_filename):
+    # We are assuming whitespace separated columns, first is mz, second is intensity, and will be marked as MS1
+    mz_list = []
+    i_list = []
+    for line in open(input_filename):
+        cleaned_line = line.rstrip()
+        if len(cleaned_line) == 0:
+            continue
+        mz, i = cleaned_line.split()
+
+        mz_list.append(float(mz))
+        i_list.append(float(i))
+        
+    ms1_df = pd.DataFrame()
+    ms1_df['mz'] = mz_list
+    ms1_df['i'] = i_list
+    ms1_df['i_norm'] = ms1_df['i'] / max(ms1_df['i'])
+    ms1_df['i_tic_norm'] = ms1_df['i'] / sum(ms1_df['i'])
+    ms1_df['scan'] = 1
+    ms1_df['rt'] = 0
+    ms1_df['polarity'] = "Positive"
+
+    print(ms1_df)
+
+    return ms1_df, pd.DataFrame()
